#include "otabrowse5.h"
#pragma hdrstop

#include "notifier.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TNotifierForm *NotifierForm;
//---------------------------------------------------------------------------
TMemo *memo;
void __fastcall log(AnsiString s)
{ if (memo!=NULL) memo->Lines->Add(s);
}
void __fastcall logdot()
{ if (memo!=NULL) memo->Text = memo->Text+".";
}

__fastcall TNotifierForm::TNotifierForm(TComponent* Owner)
        : TForm(Owner), s(NULL), m(NULL), fe(NULL), ds(NULL), num(-1)
{ mLog->Lines->Clear();
  memo=mLog;
}
__fastcall TNotifierForm::~TNotifierForm()
{ memo=NULL;
  StopNotifiers();
}
void __fastcall TNotifierForm::StopNotifiers()
{ if (num==-1)
  { if (s!=NULL) s->RemoveNotifier(num);
    if (m!=NULL) m->RemoveNotifier(num);
    if (fe!=NULL) fe->RemoveNotifier(num);
    if (ds!=NULL) ds->RemoveNotifier(num);
    num=-1;
  }
  // We've removed it. It should no longer send anything to the log.
  s=NULL; m=NULL; fe=NULL; ds=NULL;
}
//---------------------------------------------------------------------------

class TLogIDENotifier : public IOTAIDENotifier
{
private:
  long ref;
public:
  TLogIDENotifier() : IOTAIDENotifier(), ref(0) {}
  // IUnknown
  HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppv) {if (riid==IID_IUnknown || riid==IID_IOTANotifier || riid==IID_IOTAIDENotifier) {*ppv=this; AddRef(); return S_OK;} else return E_NOINTERFACE;}
  ULONG STDMETHODCALLTYPE AddRef() {return InterlockedIncrement(&ref);}
  ULONG STDMETHODCALLTYPE Release() {int tmp = InterlockedDecrement(&ref); if (tmp==0) delete this; return tmp;}
  // IOTANotifier
  void __fastcall AfterSave() {log("AfterSave");}
  void __fastcall BeforeSave() {log("BeforeSave");}
  void __fastcall Destroyed() {log("Destroyed!");}
  void __fastcall Modified() {logdot();}
  // IOTAIDENotifier
  void __fastcall FileNotification(TOTAFileNotification fn,const AnsiString FileName,bool &cancel)
  { AnsiString s;
    switch (fn)
    { case ofnFileOpening: s="FileOpening"; break;
      case ofnFileOpened: s="FileOpened"; break;
      case ofnFileClosing: s="FileClosing"; break;
      case ofnDefaultDesktopLoad: s="DefaultDesktopLoad"; break;
      case ofnDefaultDesktopSave: s="DefaultDesktopSave"; break;
      case ofnProjectDesktopLoad: s="ProjectDesktopLoad"; break;
      case ofnProjectDesktopSave: s="ProjectDesktopSave"; break;
      case ofnPackageInstalled: s="PackageInstalled"; break;
      case ofnPackageUninstalled: s="PackageUninstalled"; break;
      default: s="???";
    }
    s=s+" - "+FileName;
    log(s);
  }
  void __fastcall BeforeCompile(const _di_IOTAProject Project, bool &cancel) {log("BeforeCompile");}
  void __fastcall AfterCompile(bool succeeded) {AnsiString s="AfterCompile"; if (succeeded) s=s+" (succeeded!)"; else s=s+" (failed)"; log(s);}
};


class TLogModuleNotifier : public IOTAModuleNotifier
{
private:
  long ref;
public:
  TLogModuleNotifier() : IOTAModuleNotifier(), ref(0) {}
  // IUnknown
  HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppv) {if (riid==IID_IUnknown || riid==IID_IOTANotifier || riid==IID_IOTAModuleNotifier) {*ppv=this; AddRef(); return S_OK;} else return E_NOINTERFACE;}
  ULONG STDMETHODCALLTYPE AddRef() {return InterlockedIncrement(&ref);}
  ULONG STDMETHODCALLTYPE Release() {int tmp = InterlockedDecrement(&ref); if (tmp==0) delete this; return tmp;}
  // IOTANotifier
  void __fastcall AfterSave() {log("AfterSave");}
  void __fastcall BeforeSave() {log("BeforeSave");}
  void __fastcall Destroyed() {log("Destroyed!");}
  void __fastcall Modified() {logdot();}
  // IOTAModuleNotifier
  bool __fastcall CheckOverwrite() {log("CheckOverwrite"); return true;}
  void __fastcall ModuleRenamed(const AnsiString NewName) {log("Rename: "+NewName);}
};


class TLogFormNotifier : public IOTAFormNotifier
{
private:
  long ref;
public:
  TLogFormNotifier() : IOTAFormNotifier(), ref(0) {}
  // IUnknown
  HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppv) {if (riid==IID_IUnknown || riid==IID_IOTANotifier || riid==IID_IOTAFormNotifier) {*ppv=this; AddRef(); return S_OK;} else return E_NOINTERFACE;}
  ULONG STDMETHODCALLTYPE AddRef() {return InterlockedIncrement(&ref);}
  ULONG STDMETHODCALLTYPE Release() {int tmp = InterlockedDecrement(&ref); if (tmp==0) delete this; return tmp;}
  // IOTANotifier
  void __fastcall AfterSave() {log("AfterSave");}
  void __fastcall BeforeSave() {log("BeforeSave");}
  void __fastcall Destroyed() {log("Destroyed!");}
  void __fastcall Modified() {logdot();}
  // IOTAFormNotifier
  void __fastcall FormActivated() {log("Form activated");}
  void __fastcall FormSaving() {log("Form saving");}
  void __fastcall ComponentRenamed(void* ComponentHandle,const AnsiString OldName,const AnsiString NewName)
  { AnsiString s="'"+OldName+"' -> '"+NewName+"' @0x"+IntToHex((int)ComponentHandle,8);
    log(s);
  }
};


class TLogDebugNotifier : public IOTADebuggerNotifier
{
private:
  long ref;
public:
  TLogDebugNotifier() : IOTADebuggerNotifier(), ref(0) {}
  // IUnknown
  HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppv) {if (riid==IID_IUnknown || riid==IID_IOTANotifier || riid==IID_IOTADebuggerNotifier) {*ppv=this; AddRef(); return S_OK;} else return E_NOINTERFACE;}
  ULONG STDMETHODCALLTYPE AddRef() {return InterlockedIncrement(&ref);}
  ULONG STDMETHODCALLTYPE Release() {int tmp = InterlockedDecrement(&ref); if (tmp==0) delete this; return tmp;}
  // IOTANotifier
  void __fastcall AfterSave() {log("AfterSave");}
  void __fastcall BeforeSave() {log("BeforeSave");}
  void __fastcall Destroyed() {log("Destroyed!");}
  void __fastcall Modified() {logdot();}
  // IOTADebugNotifier
  void __fastcall ProcessCreated(_di_IOTAProcess p) {log("ProcessCreated "+p->GetProcessId());}
  void __fastcall ProcessDestroyed(_di_IOTAProcess p) {log("ProcessDestroyed "+p->GetProcessId());}
  void __fastcall BreakpointAdded(_di_IOTABreakpoint bp) {log("BreakpointAdded "+bp->GetGroupName());}
  void __fastcall BreakpointDeleted(_di_IOTABreakpoint bp) {log("BreakpointDeleted "+bp->GetGroupName());}
};



//---------------------------------------------------------------------------
void __fastcall TNotifierForm::StartServices(_di_IOTAServices as)
{ s=as;
  TLogIDENotifier *not=new TLogIDENotifier(); not->AddRef();
  num=s->AddNotifier(not); not->Release();
  Caption="ProjectNotifier - "+AnsiString(num);
  // We've now abnegated responsibility for it. The system will be the one
  // that destroys the notifier
  // However, we have 'num'. Before we close we will first free num.
  // It is expected that the notifier will never again try to log after
  // we have released it.
}
void __fastcall TNotifierForm::StartModule(_di_IOTAModule am)
{ m=am;
  TLogModuleNotifier *not=new TLogModuleNotifier(); not->AddRef();
  num=m->AddNotifier(not); not->Release();
  Caption="ModuleNotifier - "+AnsiString(num);
}
void __fastcall TNotifierForm::StartForm(_di_IOTAFormEditor afe)
{ fe=afe;
  TLogFormNotifier *not=new TLogFormNotifier(); not->AddRef();
  num=fe->AddNotifier(not); not->Release();
  Caption="FormNotifier - "+AnsiString(num);
}
void __fastcall TNotifierForm::StartDebugger(_di_IOTADebuggerServices ads)
{ ds=ads;
  TLogDebugNotifier *not=new TLogDebugNotifier(); not->AddRef();
  num=ds->AddNotifier(not); not->Release();
  Caption="DebugNotifier - "+AnsiString(num);
}


void __fastcall TNotifierForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{ StopNotifiers();
  Action=caFree;
}
//---------------------------------------------------------------------------
void __fastcall TNotifierForm::mLogDblClick(TObject *Sender)
{ mLog->Lines->Clear();
}
//---------------------------------------------------------------------------

